import { MetricServiceClient } from '@google-cloud/monitoring'

const client = new MetricServiceClient()

const projectId = 'holi-shared'
const metricTypePrefix = 'custom.googleapis.com/performance/graphql'

const createMetric = async (name) => {
  const request = {
    name: client.projectMetricDescriptorPath(projectId, `${metricTypePrefix}/${name}`),
    metricDescriptor: {
      description: `The measurements for the "${name}" request`,
      displayName: `Performance - GraphQL - ${name}`,
      type: `${metricTypePrefix}/${name}`,
      metricKind: 'GAUGE',
      valueType: 'DOUBLE',
      unit: 'ms',
      labels: [
        {
          key: 'value_description',
          valueType: 'STRING',
          description: 'The description of the value',
        },
      ],
    },
  }

  // Creates a custom metric descriptor
  return await client.createMetricDescriptor(request)
}

export const createIntMetric = async (name) => {
  const request = {
    name: client.projectMetricDescriptorPath(projectId, `${metricTypePrefix}/${name}`),
    metricDescriptor: {
      description: `The amount of ${name}`,
      displayName: `Performance - GraphQL - ${name}`,
      type: `${metricTypePrefix}/${name}`,
      metricKind: 'GAUGE',
      valueType: 'INT64',
      labels: [
        {
          key: 'value_description',
          valueType: 'STRING',
          description: 'The description of the value',
        },
      ],
    },
  }

  // Creates a custom metric descriptor
  return await client.createMetricDescriptor(request)
}

export const sendMetric = async (timestamp, name, measure, value) => {
  await client.createTimeSeries({
    name: client.projectMetricDescriptorPath(projectId, `${metricTypePrefix}/${name}`),
    timeSeries: [
      {
        metric: {
          type: `${metricTypePrefix}/${name}`,
          labels: {
            value_description: measure,
          },
        },
        resource: {
          type: 'global',
          labels: {
            project_id: projectId,
          },
        },
        points: [
          {
            interval: {
              endTime: {
                seconds: timestamp,
              },
            },
            value: value,
          },
        ],
      },
    ],
  })
}

export const appendToTimeSeries = async (timestamp, name, data) => {
  const descriptor = await createMetric(name)
  ;['avg', 'min', 'med', 'max', 'p(90)', 'p(95)', 'p(99)'].forEach(async (measure) => {
    sendMetric(timestamp, name, measure, { doubleValue: data[measure] })
  })
}
